package javafxcustom;

import javafx.scene.layout.Container;
import java.util.HashMap;
import java.lang.IllegalStateException;
import javafx.scene.Node;
import javafx.geometry.HPos;
import javafx.geometry.VPos;
import javafx.util.Math;

// A container that is similar to the AWT BorderLayout
public class FXBorder extends Container {
    public var hgap:Number = 0 on replace {
        requestLayout();
    }

    public var vgap:Number = 0 on replace {
        requestLayout();
    }

    public var nodeHPos = HPos.CENTER on replace {
        requestLayout();
    }

    public var nodeVPos = VPos.CENTER on replace {
        requestLayout();
    }

    override var content on replace {
        mapValid = false;
    }

    // Map from FXBorderPosition to node at that position.
    var mapValid = false;
    var positionMap = HashMap {};
    var hgaps = 0;
    var vgaps = 0;

    public override function getPrefWidth(height:Number) {
        buildPositionMap();
        var northWidth = getNodePrefWidth(getNodeAt(FXPosition.NORTH));
        var southWidth = getNodePrefWidth(getNodeAt(FXPosition.SOUTH));
        var eastWidth = getNodePrefWidth(getNodeAt(FXPosition.EAST));
        var westWidth = getNodePrefWidth(getNodeAt(FXPosition.WEST));
        var centerWidth = getNodePrefWidth(getNodeAt(FXPosition.CENTER));
        var middleWidth = eastWidth + centerWidth + westWidth + hgaps * hgap;
        return Math.max(northWidth, Math.max(middleWidth, southWidth));
    }

    public override function getPrefHeight(width:Number) {
        buildPositionMap();
        var eastHeight = getNodePrefHeight(getNodeAt(FXPosition.EAST));
        var westHeight = getNodePrefHeight(getNodeAt(FXPosition.WEST));
        var centerHeight = getNodePrefHeight(getNodeAt(FXPosition.CENTER));
        var northHeight = getNodePrefHeight(getNodeAt(FXPosition.NORTH));
        var southHeight = getNodePrefHeight(getNodeAt(FXPosition.SOUTH));
        return northHeight + southHeight
               + Math.max(eastHeight, Math.max(centerHeight, westHeight))
               + vgaps * vgap;
    }

    public override function getMinWidth() {
        buildPositionMap();
        var northWidth = getNodeMinWidth(getNodeAt(FXPosition.NORTH));
        var southWidth = getNodeMinWidth(getNodeAt(FXPosition.SOUTH));
        var eastWidth = getNodeMinWidth(getNodeAt(FXPosition.EAST));
        var westWidth = getNodeMinWidth(getNodeAt(FXPosition.WEST));
        var centerWidth = getNodeMinWidth(getNodeAt(FXPosition.CENTER));
        var middleWidth = eastWidth + centerWidth + westWidth + hgaps * hgap;
        return Math.max(northWidth, Math.max(middleWidth, southWidth));
    }

    public override function getMinHeight() {
        buildPositionMap();
        var eastHeight = getNodeMinHeight(getNodeAt(FXPosition.EAST));
        var westHeight = getNodeMinHeight(getNodeAt(FXPosition.WEST));
        var centerHeight = getNodeMinHeight(getNodeAt(FXPosition.CENTER));
        var northHeight = getNodeMinHeight(getNodeAt(FXPosition.NORTH));
        var southHeight = getNodeMinHeight(getNodeAt(FXPosition.SOUTH));
        return northHeight + southHeight
               + Math.max(eastHeight, Math.max(centerHeight, westHeight))
               + vgaps * vgap;
    }
    
    // Performs node layout on request.
    protected override function doLayout():Void {
        buildPositionMap();

        // Place the north node.
        var topSpace = 0.0;
        var node = getNodeAt(FXPosition.NORTH);
        if (node != null) {
            topSpace = Math.min(height, getNodePrefHeight(node));
            layoutNode(node, 0, 0, width, topSpace, nodeHPos, nodeVPos);
            topSpace += vgap;
        }

        // Place the south node.
        var bottomSpace = 0.0;
        node = getNodeAt(FXPosition.SOUTH);
        if (node != null) {
            bottomSpace = Math.min(height, getNodePrefHeight(node));
            layoutNode(node,
                       0, Math.max(0, height - bottomSpace),
                       width, bottomSpace, nodeHPos, nodeVPos);
            bottomSpace += vgap;
        }

        // Place the west and east nodes.
        var centerHeight = height - topSpace - bottomSpace;

        var westSpace = 0.0;
        node = getNodeAt(FXPosition.WEST);
        if (node != null) {
            westSpace = Math.min(width, getNodePrefWidth(node));
            layoutNode(node, 0, topSpace, westSpace, centerHeight,
                       nodeHPos, nodeVPos);
            westSpace += hgap;
        }

        var eastSpace = 0.0;
        node = getNodeAt(FXPosition.EAST);
        if (node != null) {
            eastSpace = Math.min(width, getNodePrefWidth(node));
            layoutNode(node, 
                       Math.max(0, width - eastSpace), topSpace,
                       eastSpace, centerHeight, nodeHPos, nodeVPos);
            eastSpace += hgap;
        }

        // Place the center node
        node = getNodeAt(FXPosition.CENTER);
        if (node != null) {
            layoutNode(node, westSpace, topSpace,
                       Math.max(0, width - eastSpace - westSpace),
                       Math.max(0, height - topSpace - bottomSpace),
                       nodeHPos, nodeVPos);
        }
    }

    // Builds the map from FXBorderPosition to node. Only one node is
    // allowed at each position.
    function buildPositionMap():Void {
        if (not mapValid) {
            var verticalCount = 0;
            var horizontalCount = 0;
            positionMap.clear();
            for (node in getManaged(content)) {
                var position = null;
                var l = node.layoutInfo;
                if (l instanceof FXBorderLayoutInfo) {
                    position = (l as FXBorderLayoutInfo).position;
                }
                if (position == null) {
                    position = FXPosition.CENTER;
                }
                if (positionMap.containsKey(position)) {
                    throw new IllegalStateException(
                            "More than one node with position {position}");
                }
                positionMap.put(position, node);

                if (isHorizontal(position)) {
                    horizontalCount++;
                }
                if (isVertical(position)) {
                    verticalCount++;
                }
            }
            vgaps = Math.max(0, horizontalCount - 1);
            hgaps = Math.max(0, verticalCount - 1);
            mapValid = true;
        }
    }

    // Gets the node at a given position, if there is one.
    function getNodeAt(position:FXPosition):Node {
        positionMap.get(position) as Node;
    }

    // Returns whether a border position is horizontal.
    function isHorizontal(position:FXPosition):Boolean {
        position == FXPosition.NORTH
        or position == FXPosition.CENTER
        or position == FXPosition.SOUTH
    }

    // Returns whether a border position is vertical.
    function isVertical(position:FXPosition):Boolean {
        position == FXPosition.EAST
        or position == FXPosition.CENTER
        or position == FXPosition.WEST
    }
}
