package javafxcustom;

import javafx.scene.Group;
import javafx.scene.Node;
import javafx.scene.control.Skin;
import javafx.scene.paint.Color;
import javafx.scene.shape.Polygon;
import javafx.scene.shape.Rectangle;
import javafx.scene.text.Text;
import javafx.scene.text.TextOrigin;
import javafx.scene.CustomNode;
import java.lang.Math;
import javafx.scene.effect.light.DistantLight;
import javafx.scene.effect.Lighting;
import javafx.scene.paint.Paint;

public class MediaPlayerBarSkin extends Skin {
    public var dragThumbColor = Color.RED;
    public var thumbColor = Color.GREEN;
    
    def bar = bind control as MediaPlayerBar;
    def barBehavior = bind behavior as MediaPlayerBarBehavior;
    var timeSlider:TimeSlider;
    def controlSize = 12;
    def hpad = 4;
    def vpad = 2;
    def defSliderWidth = 100;
    def textWidth = 72;
    def beforeSlider = hpad + controlSize + hpad
                       + controlSize + hpad;
    def afterSlider = hpad + textWidth + hpad;
    def fractionPlayed = bind bar.fractionPlayed on replace {
        timeSlider.setThumbPositionFromMedia(fractionPlayed);
    }
    override var behavior = MediaPlayerBarBehavior {};
    init {
        node = Group {
            var stop:Node;
            var play:Node;
            var pause:Node;
            var slider:Node;
            content: [
                stop = StopButton {
                    layoutX: hpad
                    layoutY: vpad
                    onMousePressed: function(evt) {
                        barBehavior.stopPressed();
                    }
                }
                play = PlayButton { // Play button
                    layoutX: bind stop.boundsInParent.maxX + hpad
                    layoutY: bind stop.boundsInParent.minY
                    visible: bind not bar.playing
                    onMousePressed: function(evt) {
                        barBehavior.playPressed();
                    }
                }
                pause = PauseButton {
                    layoutX: bind play.layoutX
                    layoutY: bind stop.boundsInParent.minY
                    visible: bind bar.playing
                    onMousePressed: function(evt) {
                        barBehavior.pausePressed();
                    }
                }
                timeSlider = TimeSlider {
                    layoutX: bind play.boundsInParent.maxX + hpad
                    layoutY: bind play.boundsInParent.minY
                    height: controlSize
                    width: bind bar.width - beforeSlider - afterSlider
                    thumbColor: bind thumbColor
                    dragThumbColor: bind dragThumbColor
                    fill: bind bar.normalFill
                    fractionBuffered: bind bar.fractionBuffered
                    onMoved: function(position) {
                        barBehavior.playPositionChanged(position);
                    }
                }
                Text {
                    textOrigin: TextOrigin.TOP
                    layoutX: bind bar.width - hpad - textWidth
                    layoutY: bind stop.boundsInParent.minY + 1
                    content:
                      bind "{formatTime(bar.currentTime)}"
                           "/{formatTime(bar.totalTime)}";
                    fill: bind bar.normalFill
                }
            ]
            effect: Lighting {
                light: DistantLight {
                    azimuth: 45
                    elevation: 45
                }
            }
            onMouseWheelMoved: function(evt) {
                barBehavior.mouseWheelMoved(evt.wheelRotation);
            }
        }
    }

    protected override function getPrefWidth(height:Number) {
        beforeSlider + defSliderWidth + afterSlider;
    }

    protected override function getPrefHeight(width:Number) {
        controlSize + 2 * vpad;
    }

    override function contains(x:Number, y:Number) {
        node.contains(x, y);
    }

    override function intersects(x:Number, y:Number,
                                 width:Number, height:Number) {
        node.intersects(x, y, width, height);
    }

    function formatTime(time:Duration):String {
        var minutes = time.toMinutes() as Integer;
        var seconds = (time.toSeconds() mod 60) as Integer;
        "{%02d minutes}:{%02d seconds}"
    }
}

// Internal controls.
class StopButton extends CustomNode {
    override var children =
        Rectangle {
            width: controlSize
            height: controlSize
            fill: bind if (hover and bar.playing) bar.activeFill
                       else bar.normalFill
        }
}

class PlayButton extends CustomNode {
    override var children =
        Polygon {
            points: [
                0, 0,
                controlSize, controlSize / 2,
                0, controlSize
            ]
            fill: bind if (hover and not bar.playing) bar.activeFill
                       else bar.normalFill
        }
}

class PauseButton extends CustomNode {
    override var children = {
        def fill = bind if (hover and bar.playing) bar.activeFill
                        else bar.normalFill;
        Group {
            content: [
                Rectangle {
                    x: 0
                    y: 0
                    width: controlSize / 3
                    height: controlSize
                    fill: bind fill
                }
                Rectangle {
                    x: (2 * controlSize )/ 3 - 1
                    y: 0
                    width: controlSize / 3
                    height: controlSize
                    fill: bind fill
                }
            ]
        }
    }
}

class TimeSlider extends CustomNode {
    public var thumbColor:Paint = Color.GREEN;
    public var dragThumbColor:Paint = Color.RED;
    public var fill:Paint = Color.LIGHTGRAY;
    public var width:Number;
    public var height:Number;
    public var onMoved: function(position:Number);
    public var fractionBuffered:Number;

    def thumbWidth = 4;
    def outlineWidth = 2;
    
    var thumbPosition = 0.0;
    var thumbMoving = false;

    override var children =
        Group {
            content: [
                Rectangle { // The fill
                    x: 0 y: bind outlineWidth
                    height: bind height - 2 * outlineWidth
                    width: bind width * fractionBuffered
                    fill: bind fill
                }
                Rectangle { // The outline
                    x: 0 y: bind outlineWidth
                    width: bind width
                    height: bind height - 2 * outlineWidth
                    stroke: bind fill
                    strokeWidth: bind outlineWidth
                    fill: Color.TRANSPARENT
                }
                Rectangle { // The thumb
                    x: bind thumbPosition * width - thumbWidth / 2
                    y: 0
                    width: bind thumbWidth height: bind height
                    fill: Color.TRANSPARENT
                    stroke: bind if (thumbMoving) dragThumbColor else thumbColor
                    strokeWidth: bind outlineWidth
                }
            ]
            
            onMousePressed: function(evt) {
                thumbMoving = true;
                setThumbPositionFromMouse(evt.x);
            }
            onMouseDragged: function(evt) {
                thumbMoving = true;
                setThumbPositionFromMouse(evt.x);
            }
            onMouseReleased: function(evt) {
                if (onMoved != null) {
                    onMoved(thumbPosition);
                }
                thumbMoving = false;
            }
        }

    public function setThumbPositionFromMedia(pos:Number) {
        if (not thumbMoving) {
            thumbPosition = Math.max(0, Math.min(1.0, pos));
        }
    }

    function setThumbPositionFromMouse(x:Number) {
        var pos = x - outlineWidth;
        thumbPosition = Math.max(0, Math.min(pos, width)) / width;
    }
}

